@@ -576,6 +576,14 @@ void htp_config_set_plusspace_decode(htp_cfg_t *cfg, enum htp_decoder_ctx_t ctx,
  */
 void htp_config_set_response_decompression(htp_cfg_t *cfg, int enabled);
 
+/**
+ * Controls whether compressed request bodies will be automatically decompressed.
+ *
+ * @param[in] cfg
+ * @param[in] enabled set to 1 to enable decompression, 0 otherwise
+ */
+void htp_config_set_request_decompression(htp_cfg_t *cfg, int enabled);
+
 /**
  * Configure desired server personality.
  *
