@@ -113,12 +113,12 @@ static htp_status_t htp_connp_req_receiver_send_data(htp_connp_t *connp, int is_
  * @return HTP_OK, or a value returned from a callback.
  */
 static htp_status_t htp_connp_req_receiver_set(htp_connp_t *connp, htp_hook_t *data_receiver_hook) {
-    htp_connp_req_receiver_finalize_clear(connp);
+    htp_status_t rc = htp_connp_req_receiver_finalize_clear(connp);
 
     connp->in_data_receiver_hook = data_receiver_hook;
     connp->in_current_receiver_offset = connp->in_current_read_offset;
 
-    return HTP_OK;
+    return rc;
 }
 
 /**
