@@ -132,12 +132,12 @@ htp_status_t htp_connp_res_receiver_finalize_clear(htp_connp_t *connp) {
  * @return HTP_OK, or a value returned from a callback.
  */
 static htp_status_t htp_connp_res_receiver_set(htp_connp_t *connp, htp_hook_t *data_receiver_hook) {
-    htp_connp_res_receiver_finalize_clear(connp);
+    htp_status_t rc = htp_connp_res_receiver_finalize_clear(connp);
 
     connp->out_data_receiver_hook = data_receiver_hook;
     connp->out_current_receiver_offset = connp->out_current_read_offset;
 
-    return HTP_OK;
+    return rc;
 }
 
 /**
@@ -418,8 +418,10 @@ htp_status_t htp_connp_RES_BODY_CHUNKED_LENGTH(htp_connp_t *connp) {
             connp->out_chunked_length = htp_parse_chunked_length(data, len);
 
             // empty chunk length line, lets try to continue
-            if (connp->out_chunked_length == -1004)
+            if (connp->out_chunked_length == -1004) {
+                connp->out_current_consume_offset = connp->out_current_read_offset;
                 continue;
+            }
             if (connp->out_chunked_length < 0) {
                 // reset out_current_read_offset so htp_connp_RES_BODY_IDENTITY_STREAM_CLOSE
                 // doesn't miss the first bytes
