@@ -38,6 +38,16 @@
 
 #include "htp_config_auto.h"
 
+//inet_pton
+#if _WIN32
+#include <ws2tcpip.h>
+#else // mac, linux, freebsd
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#endif
+
 #include "htp_private.h"
 
 /**
@@ -219,7 +229,7 @@ int htp_convert_method_to_number(bstr *method) {
  * @return 0 or 1
  */
 int htp_is_line_empty(unsigned char *data, size_t len) {
-    if ((len == 1) ||
+    if (((len == 1) && ((data[0] == CR) || (data[0] == LF))) ||
         ((len == 2) && (data[0] == CR) && (data[1] == LF))) {
         return 1;
     }
@@ -2442,6 +2452,17 @@ int htp_validate_hostname(bstr *hostname) {
 
     if ((len == 0) || (len > 255)) return 0;
 
+    if (data[0] == '[') {
+        // only ipv6 possible
+        if (len < 2 || len - 2 >= INET6_ADDRSTRLEN) {
+            return 0;
+        }
+        char dst[sizeof(struct in6_addr)];
+        char str[INET6_ADDRSTRLEN];
+        memcpy(str, data+1, len-2);
+        str[len-2] = 0;
+        return inet_pton(AF_INET6, str, dst);
+    }
     while (pos < len) {
         // Validate label characters.
         startpos = pos;
